'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _Filter = require('./Filter');

var _Filter2 = _interopRequireDefault(_Filter);

var _Matcher = require('./Matcher');

var _Matcher2 = _interopRequireDefault(_Matcher);

var _Parser = require('./Parser');

var _Parser2 = _interopRequireDefault(_Parser);

var _Element = require('./components/Element');

var _Element2 = _interopRequireDefault(_Element);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * @copyright   2016, Miles Johnson
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * @license     https://opensource.org/licenses/MIT
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */

/* eslint-disable react/no-unused-prop-types */

/*:: import type {
  FilterStructure,
  FilterList,
  MatcherStructure,
  MatcherList,
  ParsedNodes,
} from './types';*/


var DEFAULT_PRIORITY /*: number*/ = 100;
var globalFilters /*: FilterList*/ = [];
var globalMatchers /*: MatcherList*/ = [];

function prioritySort(a /*: MatcherStructure | FilterStructure*/, b /*: MatcherStructure | FilterStructure*/) /*: number*/ {
  return a.priority - b.priority;
}

/*:: type InterweaveProps = {
  markup: string,
  filters: Filter[],
  matchers: Matcher[],
  noHtml: boolean,
  noFilters: boolean,
  noMatchers: boolean,
  onBeforeParse: (content: string) => string,
  onAfterParse: (content: ParsedNodes) => ParsedNodes,
  tagName: string,
};*/

var Interweave = function (_React$Component) {
  _inherits(Interweave, _React$Component);

  function Interweave() {
    _classCallCheck(this, Interweave);

    return _possibleConstructorReturn(this, (Interweave.__proto__ || Object.getPrototypeOf(Interweave)).apply(this, arguments));
  }

  _createClass(Interweave, [{
    key: 'parseMarkup',


    /**
     * Parse the markup and apply hooks.
     */
    value: function parseMarkup() {
      var _props = this.props;
      var tagName = _props.tagName;
      var markup = _props.markup;
      var onBeforeParse = _props.onBeforeParse;
      var onAfterParse = _props.onAfterParse;
      var matchers = _props.matchers;
      var noMatchers = _props.noMatchers;
      var filters = _props.filters;
      var noFilters = _props.noFilters;

      var props = _objectWithoutProperties(_props, ['tagName', 'markup', 'onBeforeParse', 'onAfterParse', 'matchers', 'noMatchers', 'filters', 'noFilters']);

      var content = markup;

      if (onBeforeParse) {
        content = onBeforeParse(content);

        if (typeof content !== 'string') {
          throw new TypeError('Interweave `onBeforeParse` must return a valid HTML string.');
        }
      }

      var newMatchers = noMatchers ? [] : [].concat(_toConsumableArray(Interweave.getMatchers().map(function (row) {
        return row.matcher;
      })), _toConsumableArray(matchers));

      var newFilters = noFilters ? [] : [].concat(_toConsumableArray(Interweave.getFilters().map(function (row) {
        return row.filter;
      })), _toConsumableArray(filters));

      content = new _Parser2.default(content, props, newMatchers, newFilters).parse();

      if (onAfterParse) {
        content = onAfterParse(content);

        if (!Array.isArray(content)) {
          throw new TypeError('Interweave `onAfterParse` must return an array of strings and React elements.');
        }
      }

      return content;
    }

    /**
     * Render the component by parsing the markup.
     *
     * @returns {JSX}
     */

  }, {
    key: 'render',
    value: function render() {
      return _react2.default.createElement(
        _Element2.default,
        { tagName: this.props.tagName },
        this.parseMarkup()
      );
    }
  }], [{
    key: 'addFilter',


    /**
     * Add a filter class that will be used to cleanse HTML attributes.
     *
     * @param {Filter} filter
     * @param {Number} [priority]
     */
    value: function addFilter(filter /*: Filter*/) {
      var priority /*: number*/ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

      if (!(filter instanceof _Filter2.default)) {
        throw new TypeError('Filter must be an instance of the `Filter` class.');
      }

      // Apply and sort filters
      globalFilters.push({
        filter: filter,
        priority: priority || DEFAULT_PRIORITY + globalFilters.length
      });

      globalFilters.sort(prioritySort);
    }

    /**
     * Add a matcher class that will be used to match and replace tokens with components.
     *
     * @param {Matcher} matcher
     * @param {Number} [priority]
     */

    // eslint-disable-next-line react/sort-comp

  }, {
    key: 'addMatcher',
    value: function addMatcher(matcher /*: Matcher*/) {
      var priority /*: number*/ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

      if (!(matcher instanceof _Matcher2.default)) {
        throw new TypeError('Matcher must be an instance of the `Matcher` class.');
      }

      // Add a prop type so we can disable per instance
      Interweave.propTypes[matcher.inverseName] = _react.PropTypes.bool;

      // Append and sort matchers
      globalMatchers.push({
        matcher: matcher,
        priority: priority || DEFAULT_PRIORITY + globalMatchers.length
      });

      globalMatchers.sort(prioritySort);
    }

    /**
     * Reset all global filters.
     */

  }, {
    key: 'clearFilters',
    value: function clearFilters() {
      globalFilters = [];
    }

    /**
     * Reset all global matchers.
     */

  }, {
    key: 'clearMatchers',
    value: function clearMatchers() {
      globalMatchers = [];
    }

    /**
     * Return all defined filters for an attribute.
     *
     * @returns {{ filter: Filter }[]}
     */

  }, {
    key: 'getFilters',
    value: function getFilters() {
      return globalFilters;
    }

    /**
     * Return all defined matchers.
     *
     * @returns {{ matcher: Matcher }[]}
     */

  }, {
    key: 'getMatchers',
    value: function getMatchers() {
      return globalMatchers;
    }
  }]);

  return Interweave;
}(_react2.default.Component);

Interweave.propTypes = {
  markup: _react.PropTypes.string.isRequired,
  filters: _react.PropTypes.arrayOf(_react.PropTypes.instanceOf(_Filter2.default)),
  matchers: _react.PropTypes.arrayOf(_react.PropTypes.instanceOf(_Matcher2.default)),
  noHtml: _react.PropTypes.bool,
  noFilters: _react.PropTypes.bool,
  noMatchers: _react.PropTypes.bool,
  onBeforeParse: _react.PropTypes.func,
  onAfterParse: _react.PropTypes.func,
  tagName: _react.PropTypes.oneOf(['span', 'div', 'p'])
};
Interweave.defaultProps = {
  filters: [],
  matchers: [],
  tagName: 'span'
};
exports.default = Interweave;