'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * @copyright   2016, Miles Johnson
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * @license     https://opensource.org/licenses/MIT
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * 
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/*:: import type { MatcherFactory, MatchResponse } from './types';*/
/*:: type MatchCallback = (matches: string[]) => ({ [key: string]: any });*/

var Matcher = function () {
  function Matcher(name /*: string*/) {
    var factory /*: ?MatcherFactory*/ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

    _classCallCheck(this, Matcher);

    if (!name || name.toLowerCase() === 'html') {
      throw new Error('The matcher name "' + name + '" is not allowed.');
    }

    this.propName = name;
    this.inverseName = 'no' + (name.charAt(0).toUpperCase() + name.substr(1));
    this.customFactory = factory;
  }

  /**
   * Attempts to create a React element using a custom user provided factory,
   * or the default matcher factory.
   *
   * @param {String} match
   * @param {Object} [props]
   * @returns {ReactComponent}
   */


  _createClass(Matcher, [{
    key: 'createElement',
    value: function createElement(match /*: string*/) {
      var props /*: Object*/ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var element = null;

      if (typeof this.customFactory === 'function') {
        element = this.customFactory(match, props);
      } else {
        element = this.factory(match, props);
      }

      if (!_react2.default.isValidElement(element)) {
        throw new Error('Invalid React element created from ' + this.constructor.name + '.');
      }

      return element;
    }

    /**
     * Create a React element based on the matched token and optional props.
     *
     * @param {String} match
     * @param {Object} [props]
     * @returns {ReactComponent}
     */

  }, {
    key: 'factory',
    value: function factory(match /*: string*/) {
      var props /*: Object*/ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      throw new Error(this.constructor.name + ' must return a React element.');
    }

    /**
     * Defines the HTML tag name that the resulting React element will be.
     *
     * @returns {String}
     */

  }, {
    key: 'getTagName',
    value: function getTagName() {
      throw new Error(this.constructor.name + ' must define the HTML tag name it will render.');
    }

    /**
     * Attempt to match against the defined string.
     * Return `null` if no match found, else return the `match`
     * and any optional props to pass along.
     *
     * @param {String} string
     * @returns {Object|null}
     */

  }, {
    key: 'match',
    value: function match(string /*: string*/) {
      throw new Error(this.constructor.name + ' must define a pattern matcher.');
    }

    /**
     * Trigger the actual pattern match and package the matched
     * response through a callback.
     *
     * @param {String} string
     * @param {String|RegExp} pattern
     * @param {Function} callback
     * @returns {Object}
     */

  }, {
    key: 'doMatch',
    value: function doMatch(string /*: string*/, pattern /*: string | RegExp*/, callback /*: MatchCallback*/) {
      var matches = string.match(pattern instanceof RegExp ? pattern : new RegExp(pattern, 'i'));

      if (!matches) {
        return null;
      }

      return _extends({}, callback(matches), {
        match: matches[0]
      });
    }
  }]);

  return Matcher;
}();

exports.default = Matcher;