'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

/**
 * @copyright   2016, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 * 
 */

/* eslint-disable no-useless-escape, max-len */

// https://blog.codinghorror.com/the-problem-with-urls/
// http://www.regular-expressions.info/email.html

/*:: import type { NodeConfig } from './types';*/
/*:: type ConfigMap = { [key: string]: NodeConfig };*/


// This pattern will be used at the start or end of other patterns,
// making it easier to apply matches without capturing special chars.
/*:: type FilterMap = { [key: string]: number };*/
var ALNUM_CHAR /*: string*/ = exports.ALNUM_CHAR = '[a-z0-9]{1}';

var HASHTAG_PATTERN /*: string*/ = exports.HASHTAG_PATTERN = '#([-a-z0-9_]+)';

var URL_CHAR_PART /*: string*/ = exports.URL_CHAR_PART = 'a-z0-9-_~%!$&\'*+;:@'; // Disallow . ,
var URL_SCHEME_PATTERN /*: string*/ = exports.URL_SCHEME_PATTERN = '(https?://)?';
var URL_AUTH_PATTERN /*: string*/ = exports.URL_AUTH_PATTERN = '([' + URL_CHAR_PART + ']+@)?';
var URL_HOST_PATTERN /*: string*/ = exports.URL_HOST_PATTERN = '((?:www\\.)?' + ALNUM_CHAR + '[-a-z0-9.]*[-a-z0-9]+\\.[a-z]{2,63})';
var URL_PORT_PATTERN /*: string*/ = exports.URL_PORT_PATTERN = '(:[0-9]+)?';
var URL_PATH_PATTERN /*: string*/ = exports.URL_PATH_PATTERN = '(/[' + URL_CHAR_PART + '/]*(?:\\.[a-z]{2,8})?)?';
var URL_QUERY_PATTERN /*: string*/ = exports.URL_QUERY_PATTERN = '(\\?[' + URL_CHAR_PART + '=\\[\\]]*)?';
var URL_FRAGMENT_PATTERN /*: string*/ = exports.URL_FRAGMENT_PATTERN = '(#[\\w%/]*)?';
var URL_PATTERN /*: string*/ = exports.URL_PATTERN = [URL_SCHEME_PATTERN, URL_AUTH_PATTERN, URL_HOST_PATTERN, URL_PORT_PATTERN, URL_PATH_PATTERN, URL_QUERY_PATTERN, URL_FRAGMENT_PATTERN].join('');

var IP_V4_PART /*: string*/ = exports.IP_V4_PART = '(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)';
var IP_V4_PATTERN /*: string*/ = exports.IP_V4_PATTERN = '(' + IP_V4_PART + '\\.' + IP_V4_PART + '\\.' + IP_V4_PART + '\\.' + IP_V4_PART + ')';
var IP_PATTERN /*: string*/ = exports.IP_PATTERN = [URL_SCHEME_PATTERN, URL_AUTH_PATTERN, IP_V4_PATTERN, URL_PORT_PATTERN, URL_PATH_PATTERN, URL_QUERY_PATTERN, URL_FRAGMENT_PATTERN].join('');

var EMAIL_CLASS_PART /*: string*/ = exports.EMAIL_CLASS_PART = '[a-z0-9!#$%&?*+=_{|}~-]+';
var EMAIL_USERNAME_PATTERN /*: string*/ = exports.EMAIL_USERNAME_PATTERN = '(' + ALNUM_CHAR + EMAIL_CLASS_PART + '(?:\\.' + EMAIL_CLASS_PART + ')*' + ALNUM_CHAR + ')';
var EMAIL_PATTERN /*: string*/ = exports.EMAIL_PATTERN = EMAIL_USERNAME_PATTERN + '@' + URL_HOST_PATTERN;

// This regex is taken from the `emoji-regex` library,
// but we need it without the global modifier.
// export const EMOJI_PATTERN: string = '(?:0\u20E3|1\u20E3|2\u20E3|3\u20E3|4\u20E3|5\u20E3|6\u20E3|7\u20E3|8\u20E3|9\u20E3|#\u20E3|\\*\u20E3|\uD83C(?:\uDDE6\uD83C(?:\uDDE8|\uDDE9|\uDDEA|\uDDEB|\uDDEC|\uDDEE|\uDDF1|\uDDF2|\uDDF4|\uDDF6|\uDDF7|\uDDF8|\uDDF9|\uDDFA|\uDDFC|\uDDFD|\uDDFF)|\uDDE7\uD83C(?:\uDDE6|\uDDE7|\uDDE9|\uDDEA|\uDDEB|\uDDEC|\uDDED|\uDDEE|\uDDEF|\uDDF1|\uDDF2|\uDDF3|\uDDF4|\uDDF6|\uDDF7|\uDDF8|\uDDF9|\uDDFB|\uDDFC|\uDDFE|\uDDFF)|\uDDE8\uD83C(?:\uDDE6|\uDDE8|\uDDE9|\uDDEB|\uDDEC|\uDDED|\uDDEE|\uDDF0|\uDDF1|\uDDF2|\uDDF3|\uDDF4|\uDDF5|\uDDF7|\uDDFA|\uDDFB|\uDDFC|\uDDFD|\uDDFE|\uDDFF)|\uDDE9\uD83C(?:\uDDEA|\uDDEC|\uDDEF|\uDDF0|\uDDF2|\uDDF4|\uDDFF)|\uDDEA\uD83C(?:\uDDE6|\uDDE8|\uDDEA|\uDDEC|\uDDED|\uDDF7|\uDDF8|\uDDF9|\uDDFA)|\uDDEB\uD83C(?:\uDDEE|\uDDEF|\uDDF0|\uDDF2|\uDDF4|\uDDF7)|\uDDEC\uD83C(?:\uDDE6|\uDDE7|\uDDE9|\uDDEA|\uDDEB|\uDDEC|\uDDED|\uDDEE|\uDDF1|\uDDF2|\uDDF3|\uDDF5|\uDDF6|\uDDF7|\uDDF8|\uDDF9|\uDDFA|\uDDFC|\uDDFE)|\uDDED\uD83C(?:\uDDF0|\uDDF2|\uDDF3|\uDDF7|\uDDF9|\uDDFA)|\uDDEE\uD83C(?:\uDDE8|\uDDE9|\uDDEA|\uDDF1|\uDDF2|\uDDF3|\uDDF4|\uDDF6|\uDDF7|\uDDF8|\uDDF9)|\uDDEF\uD83C(?:\uDDEA|\uDDF2|\uDDF4|\uDDF5)|\uDDF0\uD83C(?:\uDDEA|\uDDEC|\uDDED|\uDDEE|\uDDF2|\uDDF3|\uDDF5|\uDDF7|\uDDFC|\uDDFE|\uDDFF)|\uDDF1\uD83C(?:\uDDE6|\uDDE7|\uDDE8|\uDDEE|\uDDF0|\uDDF7|\uDDF8|\uDDF9|\uDDFA|\uDDFB|\uDDFE)|\uDDF2\uD83C(?:\uDDE6|\uDDE8|\uDDE9|\uDDEA|\uDDEB|\uDDEC|\uDDED|\uDDF0|\uDDF1|\uDDF2|\uDDF3|\uDDF4|\uDDF5|\uDDF6|\uDDF7|\uDDF8|\uDDF9|\uDDFA|\uDDFB|\uDDFC|\uDDFD|\uDDFE|\uDDFF)|\uDDF3\uD83C(?:\uDDE6|\uDDE8|\uDDEA|\uDDEB|\uDDEC|\uDDEE|\uDDF1|\uDDF4|\uDDF5|\uDDF7|\uDDFA|\uDDFF)|\uDDF4\uD83C\uDDF2|\uDDF5\uD83C(?:\uDDE6|\uDDEA|\uDDEB|\uDDEC|\uDDED|\uDDF0|\uDDF1|\uDDF2|\uDDF3|\uDDF7|\uDDF8|\uDDF9|\uDDFC|\uDDFE)|\uDDF6\uD83C\uDDE6|\uDDF7\uD83C(?:\uDDEA|\uDDF4|\uDDF8|\uDDFA|\uDDFC)|\uDDF8\uD83C(?:\uDDE6|\uDDE7|\uDDE8|\uDDE9|\uDDEA|\uDDEC|\uDDED|\uDDEE|\uDDEF|\uDDF0|\uDDF1|\uDDF2|\uDDF3|\uDDF4|\uDDF7|\uDDF8|\uDDF9|\uDDFB|\uDDFD|\uDDFE|\uDDFF)|\uDDF9\uD83C(?:\uDDE6|\uDDE8|\uDDE9|\uDDEB|\uDDEC|\uDDED|\uDDEF|\uDDF0|\uDDF1|\uDDF2|\uDDF3|\uDDF4|\uDDF7|\uDDF9|\uDDFB|\uDDFC|\uDDFF)|\uDDFA\uD83C(?:\uDDE6|\uDDEC|\uDDF2|\uDDF8|\uDDFE|\uDDFF)|\uDDFB\uD83C(?:\uDDE6|\uDDE8|\uDDEA|\uDDEC|\uDDEE|\uDDF3|\uDDFA)|\uDDFC\uD83C(?:\uDDEB|\uDDF8)|\uDDFD\uD83C\uDDF0|\uDDFE\uD83C(?:\uDDEA|\uDDF9)|\uDDFF\uD83C(?:\uDDE6|\uDDF2|\uDDFC)))|[\xA9\xAE\u203C\u2049\u2122\u2139\u2194-\u2199\u21A9\u21AA\u231A\u231B\u2328\u23CF\u23E9-\u23F3\u23F8-\u23FA\u24C2\u25AA\u25AB\u25B6\u25C0\u25FB-\u25FE\u2600-\u2604\u260E\u2611\u2614\u2615\u2618\u261D\u2620\u2622\u2623\u2626\u262A\u262E\u262F\u2638-\u263A\u2648-\u2653\u2660\u2663\u2665\u2666\u2668\u267B\u267F\u2692-\u2694\u2696\u2697\u2699\u269B\u269C\u26A0\u26A1\u26AA\u26AB\u26B0\u26B1\u26BD\u26BE\u26C4\u26C5\u26C8\u26CE\u26CF\u26D1\u26D3\u26D4\u26E9\u26EA\u26F0-\u26F5\u26F7-\u26FA\u26FD\u2702\u2705\u2708-\u270D\u270F\u2712\u2714\u2716\u271D\u2721\u2728\u2733\u2734\u2744\u2747\u274C\u274E\u2753-\u2755\u2757\u2763\u2764\u2795-\u2797\u27A1\u27B0\u27BF\u2934\u2935\u2B05-\u2B07\u2B1B\u2B1C\u2B50\u2B55\u3030\u303D\u3297\u3299]|\uD83C[\uDC04\uDCCF\uDD70\uDD71\uDD7E\uDD7F\uDD8E\uDD91-\uDD9A\uDE01\uDE02\uDE1A\uDE2F\uDE32-\uDE3A\uDE50\uDE51\uDF00-\uDF21\uDF24-\uDF93\uDF96\uDF97\uDF99-\uDF9B\uDF9E-\uDFF0\uDFF3-\uDFF5\uDFF7-\uDFFF]|\uD83D[\uDC00-\uDCFD\uDCFF-\uDD3D\uDD49-\uDD4E\uDD50-\uDD67\uDD6F\uDD70\uDD73-\uDD79\uDD87\uDD8A-\uDD8D\uDD90\uDD95\uDD96\uDDA5\uDDA8\uDDB1\uDDB2\uDDBC\uDDC2-\uDDC4\uDDD1-\uDDD3\uDDDC-\uDDDE\uDDE1\uDDE3\uDDEF\uDDF3\uDDFA-\uDE4F\uDE80-\uDEC5\uDECB-\uDED0\uDEE0-\uDEE5\uDEE9\uDEEB\uDEEC\uDEF0\uDEF3]|\uD83E[\uDD10-\uDD18\uDD80-\uDD84\uDDC0]';
var EMOJI_SHORTNAME_PATTERN /*: string*/ = exports.EMOJI_SHORTNAME_PATTERN = ':[-a-z0-9_]+:';

// Parser rules for HTML tags
var PARSER_ALLOW /*: number*/ = exports.PARSER_ALLOW = 1; // Allow element and children
var PARSER_DENY /*: number*/ = exports.PARSER_DENY = 2; // Do not render this element or its children
var PARSER_PASS_THROUGH /*: number*/ = exports.PARSER_PASS_THROUGH = 3; // Do not render this element but allow its children

var TYPE_INLINE /*: string*/ = exports.TYPE_INLINE = 'inline';
var TYPE_BLOCK /*: string*/ = exports.TYPE_BLOCK = 'block';

var inlineConfig /*: NodeConfig*/ = {
  rule: PARSER_ALLOW,
  type: TYPE_INLINE,
  inline: true,
  block: false,
  self: false,
  children: []
};

var blockConfig /*: NodeConfig*/ = {
  rule: PARSER_ALLOW,
  type: TYPE_BLOCK,
  inline: true,
  block: true,
  self: true,
  children: []
};

// Tags not listed here will be denied
// https://developer.mozilla.org/en-US/docs/Web/HTML/Element
var TAGS /*: ConfigMap*/ = exports.TAGS = {
  a: _extends({}, inlineConfig),
  abbr: _extends({}, inlineConfig),
  acronym: _extends({}, inlineConfig, {
    rule: PARSER_PASS_THROUGH
  }),
  address: _extends({}, blockConfig, {
    self: false
  }),
  article: _extends({}, blockConfig),
  aside: _extends({}, blockConfig),
  audio: _extends({}, blockConfig, {
    children: ['track', 'source']
  }),
  b: _extends({}, inlineConfig),
  big: _extends({}, inlineConfig, {
    rule: PARSER_PASS_THROUGH
  }),
  blockquote: _extends({}, blockConfig),
  body: _extends({}, blockConfig, {
    rule: PARSER_PASS_THROUGH
  }),
  br: _extends({}, inlineConfig, {
    inline: false
  }),
  button: _extends({}, blockConfig),
  center: _extends({}, blockConfig, {
    rule: PARSER_PASS_THROUGH
  }),
  cite: _extends({}, inlineConfig),
  code: _extends({}, inlineConfig),
  dd: _extends({}, blockConfig),
  del: _extends({}, inlineConfig),
  details: _extends({}, blockConfig),
  dfn: _extends({}, inlineConfig),
  div: _extends({}, blockConfig),
  dl: _extends({}, blockConfig, {
    children: ['dt']
  }),
  dt: _extends({}, blockConfig, {
    children: ['dd']
  }),
  em: _extends({}, inlineConfig),
  fieldset: _extends({}, blockConfig),
  figcaption: _extends({}, blockConfig),
  figure: _extends({}, blockConfig),
  font: _extends({}, inlineConfig, {
    rule: PARSER_PASS_THROUGH
  }),
  footer: _extends({}, blockConfig, {
    self: false
  }),
  form: _extends({}, blockConfig, {
    rule: PARSER_PASS_THROUGH
  }),
  header: _extends({}, blockConfig, {
    self: false
  }),
  h1: _extends({}, blockConfig, {
    self: false
  }),
  h2: _extends({}, blockConfig, {
    self: false
  }),
  h3: _extends({}, blockConfig, {
    self: false
  }),
  h4: _extends({}, blockConfig, {
    self: false
  }),
  h5: _extends({}, blockConfig, {
    self: false
  }),
  h6: _extends({}, blockConfig, {
    self: false
  }),
  hr: _extends({}, blockConfig, {
    inline: false,
    block: false
  }),
  i: _extends({}, inlineConfig),
  img: _extends({}, inlineConfig, {
    inline: false
  }),
  ins: _extends({}, inlineConfig),
  kbd: _extends({}, inlineConfig),
  label: _extends({}, inlineConfig),
  legend: _extends({}, blockConfig),
  li: _extends({}, blockConfig, {
    self: false
  }),
  main: _extends({}, blockConfig, {
    self: false
  }),
  mark: _extends({}, inlineConfig),
  nav: _extends({}, blockConfig),
  ol: _extends({}, blockConfig, {
    children: ['li']
  }),
  output: _extends({}, inlineConfig),
  p: _extends({}, blockConfig),
  picture: _extends({}, inlineConfig, {
    children: ['source', 'img']
  }),
  pre: _extends({}, blockConfig),
  q: _extends({}, inlineConfig),
  s: _extends({}, inlineConfig),
  samp: _extends({}, inlineConfig),
  section: _extends({}, blockConfig),
  small: _extends({}, inlineConfig, {
    rule: PARSER_PASS_THROUGH
  }),
  source: _extends({}, inlineConfig, {
    inline: false
  }),
  span: _extends({}, inlineConfig, {
    self: true
  }),
  strong: _extends({}, inlineConfig),
  sub: _extends({}, inlineConfig),
  summary: _extends({}, blockConfig),
  sup: _extends({}, inlineConfig),
  table: _extends({}, blockConfig, {
    children: ['thead', 'tbody', 'tfoot', 'tr']
  }),
  tbody: _extends({}, blockConfig, {
    children: ['tr']
  }),
  td: _extends({}, blockConfig),
  tfoot: _extends({}, blockConfig, {
    children: ['tr']
  }),
  th: _extends({}, blockConfig),
  thead: _extends({}, blockConfig, {
    children: ['tr']
  }),
  time: _extends({}, inlineConfig),
  tr: _extends({}, blockConfig, {
    children: ['th', 'td']
  }),
  track: _extends({}, inlineConfig, {
    inline: false
  }),
  u: _extends({}, inlineConfig),
  ul: _extends({}, blockConfig, {
    children: ['li']
  }),
  var: _extends({}, inlineConfig),
  video: _extends({}, inlineConfig, {
    children: ['track', 'source']
  })
};

// Filters apply to HTML attributes
var FILTER_ALLOW /*: number*/ = exports.FILTER_ALLOW = 1;
var FILTER_DENY /*: number*/ = exports.FILTER_DENY = 2;
var FILTER_CAST_NUMBER /*: number*/ = exports.FILTER_CAST_NUMBER = 3;
var FILTER_CAST_BOOL /*: number*/ = exports.FILTER_CAST_BOOL = 4;

// Attributes not listed here will be denied
// https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes
var ATTRIBUTES /*: FilterMap*/ = exports.ATTRIBUTES = {
  alt: FILTER_ALLOW,
  cite: FILTER_ALLOW,
  class: FILTER_ALLOW,
  colspan: FILTER_CAST_NUMBER,
  controls: FILTER_CAST_BOOL,
  datetime: FILTER_ALLOW,
  default: FILTER_CAST_BOOL,
  disabled: FILTER_CAST_BOOL,
  dir: FILTER_ALLOW,
  height: FILTER_ALLOW,
  href: FILTER_ALLOW,
  id: FILTER_ALLOW,
  kind: FILTER_ALLOW,
  label: FILTER_ALLOW,
  lang: FILTER_ALLOW,
  loop: FILTER_CAST_BOOL,
  muted: FILTER_CAST_BOOL,
  poster: FILTER_ALLOW,
  role: FILTER_ALLOW,
  rowspan: FILTER_CAST_NUMBER,
  span: FILTER_CAST_NUMBER,
  src: FILTER_ALLOW,
  target: FILTER_ALLOW,
  title: FILTER_ALLOW,
  width: FILTER_ALLOW
};

// Attributes to camel case for React props
var ATTRIBUTES_TO_PROPS /*: { [key: string]: string }*/ = exports.ATTRIBUTES_TO_PROPS = {
  class: 'className',
  colspan: 'colSpan',
  datetime: 'dateTime',
  rowspan: 'rowSpan'
};